@@ -9,6 +9,7 @@ from scipy.interpolate import UnivariateSpline, interp1d, InterpolatedUnivariate
 from aug_sfutils import cocos
 
 logger = logging.getLogger('aug_sfutils.mapeq')
+logger.setLevel(logging.DEBUG)
 
 
 def get_nearest_index(tim_eq, tarr):
@@ -58,6 +59,7 @@ def rho2rho(eqm, rho_in, t_in=None, \
     converted radial coordinate
     """
 
+    logger.debug('Start rho2rho')
     if t_in is None:
         t_in = eqm.time
 
@@ -77,44 +79,61 @@ def rho2rho(eqm, rho_in, t_in=None, \
 
     if coord_in in ['rho_pol', 'Psi']:
         label_in = eqm.pfl
+        lbl_in = 'pfl'
     elif coord_in == 'rho_tor':
         label_in = eqm.tfl
+        lbl_in = 'tfl'
     elif coord_in in ['rho_V','r_V']:
         label_in = eqm.vol
         R0 = eqm.Rmag
+        lbl_in = 'vol'
     elif coord_in == 'r_a':
         R, _ = rhoTheta2rz(eqm, eqm.pfl, [0, np.pi], coord_in='Psi')
         label_in = (R[:, 0] - R[:, 1]).T**2
+        lbl_in = 'r_a'
     else:
         raise Exception('unsupported input coordinate')
 
     if coord_out in ['rho_pol', 'Psi']:
         label_out = eqm.pfl
+        lbl_out = 'pfl'
     elif coord_out == 'rho_tor':
         label_out = eqm.tfl
+        lbl_out = 'tfl'
     elif coord_out in ['rho_V','r_V']:
         label_out = eqm.vol
         R0 = eqm.Rmag
+        lbl_out = 'vol'
     elif coord_out == 'r_a':
         R, _ = rhoTheta2rz(eqm, eqm.pfl[unique_idx], [0, np.pi], \
                     t_in=eqm.time[unique_idx], coord_in='Psi')
         label_out = np.zeros_like(eqm.pfl)
         label_out[unique_idx, :] = (R[:, 0] - R[:, 1])**2
+        lbl_out = 'r_a'
     else:
         raise Exception('unsupported output coordinate')
 
-    PFL  = eqm.orientation*eqm.pfl
-    PSIX = eqm.orientation*eqm.psix
-    PSI0 = eqm.orientation*eqm.psi0
+    jcoco = eqm.cocos%10 - 1
+    psi_sign = -eqm.orientation*cocos.sigma['bp'][jcoco]
+    PFL  = psi_sign*eqm.pfl
+    PSIX = psi_sign*eqm.psix
+    PSI0 = psi_sign*eqm.psi0
     rho_output = np.zeros_like(rho)
 
     for i in unique_idx:
   
 # Calculate a normalized input and output flux 
-#            sort_wh = eqm.ind_sort[i][:eqm.lpfp[i]]    # Up to separatrix   
 
-        sep_out, mag_out = np.interp([PSIX[i], PSI0[i]], PFL[i], label_out[i])
-        sep_in , mag_in  = np.interp([PSIX[i], PSI0[i]], PFL[i], label_in [i])
+#        fout = interp1d(PFL[i], label_out[i], fill_value="extrapolate")
+#        (sep_out, mag_out) = fout([PSIX[i], PSI0[i]])
+        mag_out, sep_out = np.interp([PSI0[i], PSIX[i]],  PFL[i], label_out[i])
+        if lbl_in != lbl_out:
+#            fin  = interp1d(PFL[i], label_in[i] , fill_value="extrapolate")
+#            (sep_in , mag_in ) = fin( [PSIX[i], PSI0[i]])
+            mag_in, sep_in = np.interp([PSI0[i], PSIX[i]],  PFL[i], label_in[i])
+        else:
+            sep_in = sep_out
+            mag_in = mag_out
 
         if (abs(sep_out - mag_out) < 1e-4) or (abs(sep_in - mag_in) < 1e-4): #corrupted timepoint
             continue
@@ -170,6 +189,7 @@ def rho2rho(eqm, rho_in, t_in=None, \
         if coord_out  == 'Psi':
             rho_output[jt]  = rho_output[jt]**2*(eqm.psix[i] - eqm.psi0[i]) + eqm.psi0[i]
 
+    logger.debug('End rho2rho')
     return rho_output
 
 
@@ -231,14 +251,14 @@ def rz2brzt(eqm, r_in, z_in, t_in=None):
 
 # COCOS, Eq. 19, 24
     cocos_in = eqm.cocos
-    jc_in    = cocos_in%10 - 1
+    jcoco    = cocos_in%10 - 1
     ebp_in   = cocos_in//10
     sign_ip  = np.sign(np.nanmean(eqm.ipipsi))
     sign_b0  = np.sign(np.nanmean(eqm.jpol))
-    sigma_ip = sign_ip*cocos.sigma['rphiz'][jc_in]
-    sigma_b0 = sign_b0*cocos.sigma['rphiz'][jc_in]
+    sigma_ip = sign_ip*cocos.sigma['rphiz'][jcoco]
+    sigma_b0 = sign_b0*cocos.sigma['rphiz'][jcoco]
     phi_sign = sigma_b0
-    psi_sign = sigma_ip*cocos.sigma['bp'][jc_in]
+    psi_sign = sigma_ip*cocos.sigma['bp'][jcoco]
     psi_2pi  = (2.*np.pi)**ebp_in
     psi_fac = psi_sign*psi_2pi
     logger.debug('COCOS %d, Sign Bt %d, Sign Ip %d' %(cocos_in, sign_b0, sign_ip))
@@ -303,14 +323,14 @@ def Bmesh(eqm, t_in=None):
 
 # COCOS, Eq. 24
     cocos_in = eqm.cocos
-    jc_in    = cocos_in%10 - 1
+    jcoco    = cocos_in%10 - 1
     ebp_in   = cocos_in//10
     sign_ip  =  eqm.orientation
     sign_b0  = -eqm.orientation
-    sigma_ip = sign_ip*cocos.sigma['rphiz'][jc_in]
-    sigma_b0 = sign_b0*cocos.sigma['rphiz'][jc_in]
+    sigma_ip = sign_ip*cocos.sigma['rphiz'][jcoco]
+    sigma_b0 = sign_b0*cocos.sigma['rphiz'][jcoco]
     phi_sign = sigma_b0
-    psi_sign = cocos.sigma['bp'][jc_in]
+    psi_sign = cocos.sigma['bp'][jcoco]
     psi_2pi  = (2.*np.pi)**ebp_in
     psi_fac = psi_sign*psi_2pi
     logger.info('COCOS %d' %cocos_in)
@@ -397,7 +417,7 @@ def rz2rho(eqm, r_in, z_in, t_in=None, coord_out='rho_pol', extrapolate=True):
     offset  = np.array([eqm.Rmesh[0], eqm.Zmesh[0]])
     
     unique_idx, index_idx =  get_nearest_index(eqm.time, tarr)
-  
+
     for i in unique_idx:
         jt = index_idx[i]
         coords = np.array((r_in[jt], z_in[jt]))
@@ -405,10 +425,8 @@ def rz2rho(eqm, r_in, z_in, t_in=None, coord_out='rho_pol', extrapolate=True):
         Psi[jt] =  map_coordinates(eqm.pfm[:, :, i], index, mode='nearest',
                                    order=2, prefilter=True)
 
-    t1 = time.time()
     rho_out = rho2rho(eqm, Psi, t_in=t_in, extrapolate=extrapolate, \
               coord_in='Psi', coord_out=coord_out)
-    t2 = time.time()
 
     return rho_out
 
@@ -509,7 +527,7 @@ def rho2rz(eqm, rho_in, t_in=None, coord_in='rho_pol', all_lines=False):
     return Rsurf, zsurf
 
 
-def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='rho_pol'):
+def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, line_m=2, theta_in=0, t_in=None, coord_in='rho_pol'):
 
     """
     Computes intersections of a line with any flux surface.
@@ -517,9 +535,9 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
     Input:
     ----------
     eqm: equilibrium object
-    rho: float
+    rho: float or 1D array(nt)
         coordinate of the desired flux surface
-    t_in: float or 1darray
+    t_in: float or 1darray(nt)
         time (eqm.time in case of None)
     r_in: float
         R position of the point
@@ -542,37 +560,38 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
         t_in = eqm.time
 
     tarr = np.atleast_1d(t_in)
-    rho  = np.float32(rho)
+    rho  = np.atleast_1d(rho)
     r_in = np.float32(r_in)
     z_in = np.float32(z_in)
 
     unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
-    line_m = 3. # line length: 6 m
     n_line = int(200*line_m) + 1 # 1 cm, but then there's interpolation!
     t = np.linspace(-line_m, line_m, n_line, dtype=np.float32)
 
     line_r = t*np.cos(theta_in) + r_in
     line_z = t*np.sin(theta_in) + z_in
+
     rho_line = rz2rho(eqm, line_r, line_z, t_in=eqm.time[unique_idx], \
                            coord_out=coord_in, extrapolate=True)
 
-    if coord_in == 'Psi':
-        rho_line *= eqm.orientation
-        rho      *= eqm.orientation
     nt_in = len(tarr)
-    Rout = np.zeros((nt_in, 2), dtype=np.float32)
-    zout = np.zeros((nt_in, 2), dtype=np.float32)
+    Rout = np.nan*np.ones((nt_in, 2), dtype=np.float32)
+    zout = np.nan*np.ones((nt_in, 2), dtype=np.float32)
     for i, ii in enumerate(unique_idx):
         jt = index_idx[ii]
-
-        ind_gt = (rho_line[i] > rho)
-        ind_cross = [j for j, x in enumerate(np.diff(ind_gt)) if x]
+        if len(rho) == 1:
+            rho_ref = rho[0]
+        else:
+            rho_ref = rho[jt[0]]
+        ind_cross = np.argwhere(np.diff(np.sign(rho_line[i] - rho_ref))).flatten()
 
         if len(ind_cross) > 2:
-            logger.warning('More than 2 intersections, problems')
-            zout[jt, :] = 0
-            Rout[jt, :] = 0
+            logger.warning('More than 2 intersections at t=%.3f', tarr[jt[0]])
+            continue
+
+        if len(ind_cross) == 0:
+            logger.warning('No intersections found at t=%.3f', tarr[jt[0]])
             continue
 
         pos = 0
@@ -581,17 +600,17 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
                 ind = [j, j + 1]
             else:
                 ind = [j + 1, j]
-            ztmp = np.interp(rho, rho_line[i, ind], line_z[ind])
+            ztmp = np.interp(rho_ref, rho_line[i, ind], line_z[ind])
             if ztmp >= eqm.Zunt[ii] and ztmp <= eqm.Zoben[ii]:
                 zout[jt, pos] = ztmp
-                Rout[jt, pos] = np.interp(rho, rho_line[i, ind], line_r[ind])
+                Rout[jt, pos] = np.interp(rho_ref, rho_line[i, ind], line_r[ind])
                 pos += 1
     del line_r, line_z, rho_line
 
     return Rout, zout
 
 
-def cross_sep(eqm, r_in=1.65, z_in=0, theta_in=0, t_in=None):
+def cross_sep(eqm, r_in=1.65, z_in=0, line_m=2., theta_in=0, t_in=None):
 
     """
     Computes intersections of a line with any flux surface.
@@ -618,51 +637,12 @@ def cross_sep(eqm, r_in=1.65, z_in=0, theta_in=0, t_in=None):
 
     if t_in is None:
         t_in = eqm.time
-
     tarr = np.atleast_1d(t_in)
-    r_in = np.float32(r_in)
-    z_in = np.float32(z_in)
-
     unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
-    line_m = 3. # line length: 6 m
-    n_line = int(200*line_m) + 1 # 1 cm, but then there's interpolation!
-    t = np.linspace(-line_m, line_m, n_line, dtype=np.float32)
+    psi_sep = eqm.pfl[unique_idx, max(eqm.lpfp)]
 
-    line_r = t*np.cos(theta_in) + r_in
-    line_z = t*np.sin(theta_in) + z_in
-    psi_line = rz2rho(eqm, line_r, line_z, t_in=eqm.time[unique_idx], \
-                           coord_out='Psi', extrapolate=True)
-
-    psi_line *= eqm.orientation
-    psi_sep = eqm.pfl[:, max(eqm.lpfp)]*eqm.orientation
-    nt_in = len(tarr)
-    Rout = np.zeros((nt_in, 2), dtype=np.float32)
-    zout = np.zeros((nt_in, 2), dtype=np.float32)
-    for i, ii in enumerate(unique_idx):
-        jt = index_idx[i]
-        ind_gt = (psi_line[i] > psi_sep[ii])
-        ind_cross = [j for j, x in enumerate(np.diff(ind_gt)) if x]
-        if len(ind_cross) > 2:
-            logger.warning('More than 2 intersections, problems')
-            zout[jt, :] = 0
-            Rout[jt, :] = 0
-            continue
-
-        pos = 0
-        for j in ind_cross:
-            if psi_line[i, j + 1] > psi_line[i, j]:
-                ind = [j, j + 1]
-            else:
-                ind = [j + 1, j]
-            ztmp = np.interp(psi_sep[ii], psi_line[i, ind], line_z[ind])
-            if ztmp >= eqm.Zunt[ii] and ztmp <= eqm.Zoben[ii]:
-                zout[jt, pos] = ztmp
-                Rout[jt, pos] = np.interp(psi_sep[ii], psi_line[i, ind], line_r[ind])
-                pos += 1
-    del line_r, line_z, psi_line
-
-    return Rout, zout
+    return cross_surf(eqm, rho=psi_sep, r_in=r_in, z_in=z_in, line_m=line_m, theta_in=theta_in, t_in=t_in, coord_in='Psi')
 
 
 def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
@@ -705,7 +685,7 @@ def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
 
     unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
-#        n_line = 201 <=> 5 mm, but then there's interpolation!
+# n_line = 201 <=> 5 mm, but then there's interpolation!
 
     line_r = np.empty((len(unique_idx), ntheta, n_line))
     line_z = np.empty((len(unique_idx), ntheta, n_line))
@@ -726,10 +706,11 @@ def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
     R = np.empty((nt_in, ntheta) + rho.shape[1:], dtype=np.float32)
     z = np.empty((nt_in, ntheta) + rho.shape[1:], dtype=np.float32)
 
+    jcoco = eqm.cocos%10 - 1
     if coord_in == 'Psi':
         rho_line[:,:,0] = eqm.psi0[unique_idx][:,None]
-        rho_line *= eqm.orientation
-        rho      *= eqm.orientation
+        rho_line *= -eqm.orientation*cocos.sigma['bp'][jcoco]
+        rho      *= -eqm.orientation*cocos.sigma['bp'][jcoco]
     else:
         #solve some issues very close to the core
         rho_line[:,:,0] = 0
@@ -889,7 +870,7 @@ def to_geqdsk(eqm, t_in=0.):
 
     equ = cocos.coco2coco(eqm, cocos_out=1)
 
-    jt = np.argmin(equ.time - t_in)
+    jt = np.argmin(np.abs(equ.time - t_in))
 
     psi_in  = equ.pfl[jt]
 
@@ -919,7 +900,7 @@ def to_geqdsk(eqm, t_in=0.):
     geq['BCENTR']  = equ.B0[jt]
     geq['RCENTR']  = equ.R0  ##[jt]   # TODO this seems to be time-independent?
     geq['ZMID']    = 0.5*(equ.Zmesh[-1] + equ.Zmesh[0])
-    geq['PSIRZ']   = equ.pfm[jt]
+    geq['PSIRZ']   = equ.pfm[:, :, jt]
 
     # Interpolate onto an equidistant Psi-grid
     psi_grid = np.linspace(psi_in[0], psi_in[-1], len(equ.Rmesh))
