@@ -1,3 +1,5 @@
+"""Editing shotfile headers"""
+
 import shutil, os, logging
 import ctypes as ct
 import numpy as np
@@ -38,6 +40,8 @@ class SFH:
         return err
 
     def Mdarea(self, obj, nsteps, nx, ny, nz):
+        """Modifes the size of an AreaBase object
+        """
 
         bobj  = str_byt.to_byt(obj)
         c_obj = ct.c_char_p(bobj)
@@ -50,6 +54,8 @@ class SFH:
         libsfh.sfherror(err, 'mdarea')
 
     def Mdindex(self, obj, nx, ny, nz, qual=False):
+        """Modifes the spatial index of an object
+        """
 
         bobj  = str_byt.to_byt(obj)
         c_obj = ct.c_char_p(bobj)
@@ -64,6 +70,8 @@ class SFH:
         libsfh.sfherror(err, 'mdindex24')
 
     def Modtim(self, obj, nt):
+        """Modifes the size of the time dimension of an object
+        """
 
         bobj  = str_byt.to_byt(obj)
         c_obj = ct.c_char_p(bobj)
@@ -82,6 +90,8 @@ class SFH:
         libsfh.sfherror(err, 'modindex1')
 
     def Modsgr(self, obj, dims, qual=False):
+        """Modifes the dimenskions of a SigGroup
+        """
 
         nx = 1 + np.zeros(4, dtype=int)
         for jdim in range(len(dims)):
@@ -90,6 +100,8 @@ class SFH:
         self.Mdindex(obj, nx[1], nx[2], nx[3], qual=qual)
 
     def Modpar(self, ps, pn, dat):
+        """Modifies a parameter's size in ParSet
+        """
 
         pss = str_byt.to_str(ps)
         pns = str_byt.to_str(pn)
@@ -127,6 +139,9 @@ class SFH:
 
 
     def newobj(self, objname, objtyp, subtyp):
+        """Creates a new SFH object
+        """
+
 # objtyp: 6 SGR, 7 Signal, 8 Timebase, 13 Areabase
         objname = str_byt.to_byt(objname)
         c_obj = ct.c_char_p(objname)
@@ -137,6 +152,8 @@ class SFH:
 
 
     def newrel(self, objname, newrelname):
+        """Adds a new relation to a given object
+        """
 
         objname    = str_byt.to_byt(objname)
         newrelname = str_byt.to_byt(newrelname)
@@ -147,23 +164,27 @@ class SFH:
 
 
     def newrelt(self, objname, newreltb):
+        """Adds a new TimeBase relation to a given object
+        """
 
         objname  = str_byt.to_byt(objname)
         newreltb = str_byt.to_byt(newreltb)
         c_obj   = ct.c_char_p(objname)
         c_reltb = ct.c_char_p(newreltb)
         err = libsfh.sfhstreltb(self.c_sfid, c_obj, c_reltb)
-        libsfh.sfherror(err, 'newrel')
+        libsfh.sfherror(err, 'newrelt')
 
 
     def set_text(self, objname, newtext):
+        """Replaces the description string of a given object
+        """
 
         objname = str_byt.to_byt(objname)
         newtext = str_byt.to_byt(newtext)
         c_obj  = ct.c_char_p(objname)
         c_text = ct.c_char_p(newtext)
         err = libsfh.sfhmtext(self.c_sfid, c_obj, c_text)
-        libsfh.sfherror(err, 'newrel')
+        libsfh.sfherror(err, 'set_text')
 
 #===============
 #    New Methods
