@@ -1,11 +1,6 @@
 import numpy as np
 
 
-class EMPTY:
-
-    pass
-
-
 class SFOBJ(np.ndarray):
     """
     Subclass of numpy.ndarray adding a few metadata,
